import logging
import os
import sys
from datetime import datetime, timedelta

from src.utils.reporter import MetricsClient
from src.utils.utils import get_athena_query_results , convert_statistics_csv, org_data_to_S3

logging.basicConfig(stream=sys.stdout, level=logging.INFO)

CDCA_DATABASE = os.getenv("CDCA_DATABASE", "cdca")
CDCA_QUERY_RESULT = os.getenv("CDCA_QUERY_RESULT", "cdca-policy-suggest-analytics-dev-athena-outputs")
ORG_MONTHLY_DATA_BUCKET = os.getenv("ORG_MONTHLY_BUCKET", "kcdev01-mdr-reporting-job-infra-daily-alert")
ENV_CATEGORY = os.getenv("ENV_CATEGORY", "int")
ENV = os.getenv("ENV", "kcdev01")
PROMETHEUS_METRICS_PORT = os.environ.get("PROMETHEUS_PORT", "9000")
TIME_ZONE = os.getenv("TIME_ZONE", "America/New_York")
ALERT_TABLE = os.getenv("ALERT_TABLE", "flink_alert_mdr_prod")


def get_last_month():
    # Get the current date
    current_date = datetime.now()

    # Calculate the first day of the current month
    first_day_of_current_month = current_date.replace(day=1)

    # Subtract one day from the first day of the current month to get the last day of the previous month
    last_day_of_last_month = first_day_of_current_month - timedelta(days=1)

    # Format the date to "YYYY-MM"
    return last_day_of_last_month.strftime("%Y-%m")


def get_monthly_assets_query():
    if ENV_CATEGORY != 'prd':
        with open('./src/sqls/dev/monthly_query.sql') as monthly_query_file:
            return monthly_query_file.read().format(time_zone=TIME_ZONE)
    else:
        with open('./src/sqls/prod/monthly_query.sql') as monthly_query_file:
            return monthly_query_file.read().format(time_zone=TIME_ZONE, alert_table=ALERT_TABLE)


def main():
    metric_client = MetricsClient('mdr_reporting_job', PROMETHEUS_METRICS_PORT)
    query = get_monthly_assets_query()
    results = get_athena_query_results(database=CDCA_DATABASE, query=query, query_bucket=CDCA_QUERY_RESULT)
    org_objects = {}
    convert_statistics_csv(results, org_objects, metric_client, 'monthly')
    org_data_to_S3(org_objects, bucket=ORG_MONTHLY_DATA_BUCKET, file_name=get_last_month())


if __name__ == "__main__":
    main()
