WITH unified_orgs AS (
    -- Get orgs with MDR Unified Console enabled yesterday.
SELECT
        org_key,
        org_name
        FROM (VALUES
            ('252TYD3XM2', 'test-mdr-e2e-10apr.com'),
            ('8XMTD9YVYE', 'harshal-mdr-bb.com'),
            ('JY9T5D63VW', 'test-mdr-e2e-30jan.com')) AS t(org_key, org_name)
),
mdr_alerts AS (
    WITH mdr_alerts_all AS (
       SELECT
         triggered_alert1_alert_id_guid alert_id
       , environment
       , org_id
       , org_key
       , triggered_alert1_threat_score severity
       , device_name
       , CAST(device_id AS bigint) device_id
       , actor_process_filename app_name
       , (triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}') create_timestamp
       , "date"(triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}') create_date,
       CASE
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('TRUE_POSITIVE', 'NOT_ENOUGH_INFO')
           THEN 'Likely Threat'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('FALSE_POSITIVE')
           THEN 'Unlikely threat'
         WHEN (
           triggered_alert[1].user_state_details.workflow_state.status = 'CLOSED'
           AND (
             (triggered_alert[1].mdr_state_details.classification.last_update_timestamp < (DATE '2022-01-01') OR triggered_alert[1].mdr_state_details.classification.last_update_timestamp IS NULL)
             OR (triggered_alert[1].user_state_details.workflow_state.last_update_timestamp <= triggered_alert[1].mdr_state_details.classification.last_update_timestamp)
           )
         )
           THEN 'Closed by customer'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('NOT_REVIEWED')
           THEN 'Not reviewed'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('ALERT_CLASSIFICATION_UNKNOWN')
           THEN null
         ELSE null END
       AS determination
       , (triggered_alert[1].mdr_state_details.workflow_state.last_update_timestamp AT TIME ZONE '{time_zone}') as mdr_last_timestamp
       FROM
         (cdca.flink_alert_mdr_stage
       LEFT JOIN product_metrics.cbc_orgs USING (org_key))
       WHERE ((triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}' >= (date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}') - INTERVAL  '7' DAY))
                  AND (triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}' < date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}')))
    ),
    mdr_alerts_updates AS (
    -- Get Alerts' update
       SELECT
           triggered_alert1_alert_id_guid AS alert_id,
           CASE
              WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('TRUE_POSITIVE', 'NOT_ENOUGH_INFO')
                THEN 'Likely Threat'
              WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('FALSE_POSITIVE')
                THEN 'Unlikely threat'
              WHEN (
                triggered_alert1_user_state_details_workflow_state_status = 'CLOSED'
                AND (
                  (triggered_alert1_mdr_state_details_classification_last_update_timestamp < (DATE '2022-01-01') OR triggered_alert1_mdr_state_details_classification_last_update_timestamp IS NULL)
                  OR (triggered_alert1_user_state_details_workflow_state_last_update_timestamp <= triggered_alert1_mdr_state_details_classification_last_update_timestamp)
                )
              )
                THEN 'Closed by customer'
              WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('NOT_REVIEWED')
                THEN 'Not reviewed'
              WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('ALERT_CLASSIFICATION_UNKNOWN')
                THEN 'None'
              ELSE 'Unknown' END
            AS determination,
            triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp AT TIME ZONE '{time_zone}' AS mdr_last_timestamp
       FROM (
        select
            triggered_alert1_mdr_state_details_classification_alert_classification,
            triggered_alert1_mdr_state_details_classification_last_update_timestamp,
            triggered_alert1_user_state_details_workflow_state_status,
            triggered_alert1_user_state_details_workflow_state_last_update_timestamp,
            triggered_alert1_alert_id_guid,
            triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp,
                        row_number() over (
            partition by triggered_alert1_alert_id_guid
            order by triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp desc
       ) as seqnum
                from cdca.flink_eedr_alert_user_update_history_stage
                where triggered_alert1_alert_id_guid in (SELECT distinct alert_id FROM mdr_alerts_all)
                  and triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp is not null
       ) where seqnum = 1)
    , mdr_alerts AS (
       SELECT
         alert_id
       , environment
       , org_id
       , org_key
       , severity
       , device_name
       , device_id
       , app_name
       , create_timestamp
       , create_date
       , CASE WHEN COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp) IS NOT NULL
                       AND COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp) > create_timestamp
           THEN date_diff('second', create_timestamp, COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp))
           ELSE 0 END AS full_ttr
       , COALESCE(mdr_alerts_updates.determination, mdr_alerts_all.determination, 'None') as determination
       , 'unified' source
       FROM
         mdr_alerts_all LEFT JOIN mdr_alerts_updates USING (alert_id)
    )
    SELECT * FROM mdr_alerts
),
likely_ttr AS (
    -- Get TTR from likely alerts for for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as likely_avg_ttr,
        COUNT(*) as likely_count
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date
),
mdr_ttr AS (
    -- Get TTR from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as mdr_avg_ttr,
        COUNT(*) as mdr_count
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date
),
likely_alert_stat AS (
    -- Get alert severity distribution from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS likely_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS likely_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS likely_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS likely_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS likely_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS likely_alert_10
    FROM mdr_alerts
    WHERE determination = 'Likely Threat'
    GROUP BY org_key, create_date
),
mdr_alert_stat AS (
    -- Get alert severity distribution from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS mdr_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS mdr_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS mdr_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS mdr_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS mdr_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS mdr_alert_10
    FROM mdr_alerts
    WHERE severity >= 5
    GROUP BY org_key, create_date
),
likely_app_freq AS (
    -- Get rank of application and its name from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS likely_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS likely_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS likely_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS likely_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS likely_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS likely_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS likely_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS likely_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS likely_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS likely_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS likely_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS likely_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
mdr_app_freq AS (
    -- Get rank of application and its name from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS mdr_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS mdr_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS mdr_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS mdr_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS mdr_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS mdr_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS mdr_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS mdr_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS mdr_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS mdr_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS mdr_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS mdr_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
likely_dev_freq AS (
        -- Get rank of device and its name from likely alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS likely_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS likely_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS likely_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS likely_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS likely_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS likely_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS likely_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS likely_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS likely_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS likely_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS likely_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS likely_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
),
mdr_dev_freq AS (
        -- Get rank of device and its name from MDR alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS mdr_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS mdr_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS mdr_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS mdr_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS mdr_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS mdr_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS mdr_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS mdr_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS mdr_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS mdr_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS mdr_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS mdr_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
)
-- Group alert statistic data by org_key and create_date
SELECT
    org_key,
    unified_orgs.org_name,
    COALESCE(create_date, CURRENT_DATE) as create_date,
    likely_alert_stat.*,
    mdr_alert_stat.*,
    COALESCE(lt.likely_avg_ttr, 0) as likely_avg_ttr,
    COALESCE(lt.likely_count, 0) as likely_count,
    COALESCE(mt.mdr_avg_ttr, 0) as mdr_avg_ttr,
    COALESCE(mt.mdr_count, 0) as mdr_count,
    likely_app.*,
    mdr_app_freq.*,
    likely_dev.*,
    mdr_dev_freq.*
FROM unified_orgs
LEFT JOIN mdr_alert_stat using (org_key)
LEFT JOIN likely_alert_stat using (org_key, create_date)
LEFT JOIN likely_ttr lt using (org_key, create_date)
LEFT JOIN mdr_ttr mt using (org_key, create_date)
LEFT JOIN likely_app_freq likely_app using (org_key, create_date)
LEFT JOIN mdr_app_freq using (org_key, create_date)
LEFT JOIN likely_dev_freq likely_dev using (org_key, create_date)
LEFT JOIN mdr_dev_freq using (org_key, create_date)