SELECT 
    environment,
    org_id,
    org_key,
    org_endpoints,
    customer_endpoints,
    account_industry,
    -- Determine the size of a customer by their total endpoints, not just this single org
    CASE 
      WHEN customer_endpoints < 500 THEN 'Small'
      WHEN customer_endpoints < 5000 THEN 'Medium'
      ELSE 'Large' END as org_size
    FROM (
      WITH 
      -- One row per org
      -- Org must have at least 1 registered endpoint
      orgs AS (
        SELECT 
          distinct org_key,
          org_name,
          environment,
          org_id,
          mode
          FROM product_metrics.cbc_orgs_config
          WHERE sh_date = (SELECT max(sh_date) FROM product_metrics.cbc_orgs_config WHERE sh_date > substr(cast((CURRENT_DATE - INTERVAL '14' DAY) as varchar), 1, 10))
      )

      -- What's enabled in the org?
      -- One row per org key
--       ,org_configs AS(
--         SELECT
--           org_key,
--           -- cbDefense setting defaults to true. So we want to assume it's on, unless it's explicitly set to off
--           min(CASE WHEN flags.feature = 'cbDefense' AND flags.enabled = false THEN false ELSE true END) as "Endpoint Standard Enabled",
--           max(CASE WHEN flags.feature = 'pscResponse' AND flags.enabled = true THEN true ELSE false END) as "Enterprise EDR Enabled",
--           max(CASE WHEN flags.feature = 'xdr' AND flags.enabled = true THEN true ELSE false END) as "XDR Enabled",
--           max(CASE WHEN flags.feature = 'liveQuery'AND flags.enabled = true THEN true ELSE false END) as "Audit & Remediation Enabled",
--           max(CASE WHEN flags.feature = 'workloadsManagement' AND flags.enabled = true THEN true ELSE false END) as "Workload Enabled",
--           max(CASE WHEN flags.feature = 'endpointsVulnerabilityAssessment' AND flags.enabled = true THEN true ELSE false END) as "Vulnerability Enabled",
--           max(CASE WHEN flags.feature = 'threatSight' AND flags.enabled = true THEN true ELSE false END) as "Managed Detection Enabled",
--           max(CASE WHEN flags.feature = 'managedDetectionResponse' AND flags.enabled = true THEN true ELSE false END) as "Managed Detection & Response Enabled",
--           max(CASE WHEN flags.feature = 'mdrForEEDR' AND flags.enabled = true THEN true ELSE false END) as "Managed Threat Hunting Enabled",
--           max(CASE WHEN flags.feature = 'ngav' AND flags.enabled = true THEN true ELSE false END) as "Prevention Enabled",
--           max(CASE WHEN flags.feature = 'hostBasedFirewall' AND flags.enabled = true THEN true ELSE false END) as "HBFW Enabled",
--           max(CASE WHEN flags.feature = 'guardrails' AND flags.enabled = true THEN true ELSE false END) as "Container Hardening Enabled",
--           max(CASE WHEN flags.feature = 'kubernetesruntimeprotection' AND flags.enabled = true THEN true ELSE false END) as "Container Runtime Enabled",
--           max(CASE WHEN flags.feature = 'cisbenchmark' AND flags.enabled = true THEN true ELSE false END) as "CIS Benchmark Enabled"
--
--
--         FROM product_metrics.cbc_orgs_config
--         CROSS JOIN unnest(flags_enabled) as t(flags)
--         WHERE
--           sh_date = (SELECT max(sh_date) from (select sh_date from product_metrics.cbc_orgs_config where sh_date >= substr(cast(date_add('week', -1, now()) as varchar), 1, 10) group by sh_date having count(1) > 40000))
--         GROUP BY
--           org_key
--       ),

      -- Device Counts per org
      -- One row per org key
      ,cbc_devices AS (
        SELECT
          orgs.org_key,
          max(registered_total_active) as org_endpoints,
          max(registered_active_windows) as org_windows_endpoints,
          max(registered_active_mac) as org_mac_endpoints,
          max(registered_active_linux) as org_linux_endpoints,
          max(total_consumed_last_day) as org_endpoints_consumed
        FROM product_metrics.cbc_device_stats devices 
        INNER JOIN product_metrics.cbc_orgs orgs
        ON orgs.environment = devices.environment AND orgs.org_id = devices.org_id
        WHERE sh_date = (SELECT max(sh_date) from product_metrics.cbc_device_stats WHERE sh_date > substr(cast((CURRENT_DATE - INTERVAL '14' DAY) as varchar), 1, 10))
        GROUP BY orgs.org_key
      )
      -- What has the customer purchased?
      -- One row per org key
      -- Only paying customers
       ,customers AS (
         SELECT
           customer_org_key as org_key,
--           -- Filter out known non-customers
--           max(CASE WHEN
--             account_name IN ('Carbon Black, Inc.') THEN false
--             ELSE true END)
--           as "Is Customer",
           max(account_name) as account_name
--           max(account_sfid) as account_sfid,
--           max(concat('https://bit9.my.salesforce.com/', account_sfid)) as sf_link,
--           -- approx_distinct(customer_org_key) OVER (partition by account_sfid) as customer_org_count,
--           max(registered_total_active) as customer_endpoints,
--           max(total_consumed_last_day) as customer_endpoints_consumed,
--           max(vmstar_sales_classification) as vmstar_sales_classification,
--           max(account_industry) as account_industry,
--           max(CASE WHEN
--             contains(item_master_product, 'Cb Defense')
--             OR contains(item_master_product, 'Carbon Black Endpoint Advanced')
--             OR contains(item_master_product, 'Carbon Black Endpoint Enterprise')
--             OR contains(item_master_product, 'Carbon Black Endpoint Standard')
--             OR (contains(item_master_product, 'Carbon Black Workload') AND (sku like '%WLAD%' OR sku like '%WLEN%'))
--            THEN true ELSE false END) AS "Endpoint Standard Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Cb ThreatHunter')
--             OR contains(item_master_product, 'Carbon Black Endpoint Enterprise')
--             OR (contains(item_master_product, 'Carbon Black Workload') AND sku like '%WLEN%')
--            THEN true ELSE false END) AS "Enterprise EDR Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Cb ThreatSight')
--             AND sku like '%MDR%'
--            THEN true ELSE false END) AS "Managed Detection & Response Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Cb ThreatSight')
--             AND (sku like '%WLMD-%' OR sku like '%CBMD-%' OR sku like '%NG-MAS-%')
--            THEN true ELSE false END) AS "Managed Detection Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Carbon Black Prevention')
--            THEN true ELSE false END) AS "Prevention Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Carbon Black Endpoint Enterprise')
--             OR contains(item_master_product, 'Cb LiveOps')
--             OR contains(item_master_product, 'Carbon Black Endpoint Advanced')
--             OR (contains(item_master_product, 'Carbon Black Workload') AND (sku like '%WLAD%' OR sku like '%WLEN%'))
--            THEN true ELSE false END) AS "Audit & Remediation Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Carbon Black Workload')
--              AND (sku like '%WLAD%' OR sku like '%WLEN%' OR sku like '%WLES%')
--            THEN true ELSE false END) AS "Workload Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Carbon Black Workload')
--              AND (sku like '%CNAD%' OR sku like '%CNES%')
--            THEN true ELSE false END) AS "Container Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Host Based Firewall')
--             THEN true ELSE false END) AS "HBFW Purchased",
--           max(CASE WHEN
--             contains(item_master_product, 'Cb ThreatHunter') AND lower(sku) like '%xdr%'
--             THEN true else false END) AS "XDR Purchased"
         FROM product_metrics.pum_cbc_report_last_day
--         CROSS JOIN unnest(org_key) as t(customer_org_key)
--         CROSS JOIN unnest(subscription_sku) as t(sku)
         GROUP BY customer_org_key
--       )

    SELECT
      org_key,
      environment,
      org_id,
      orgs.*,
      org_configs.*,
      COALESCE(cbc_devices.org_endpoints, 0) as org_endpoints,
      COALESCE(cbc_devices.org_endpoints_consumed, 0) as org_endpoints_consumed,
      CASE
        WHEN COALESCE(cbc_devices.org_endpoints, 0) < 1 THEN '0'
        WHEN cbc_devices.org_endpoints < 10 THEN '1 - 10'
        WHEN cbc_devices.org_endpoints < 100 THEN '10 - 100'
        WHEN cbc_devices.org_endpoints < 1000 THEN '100 - 1000'
        WHEN cbc_devices.org_endpoints < 10000 THEN '1000 - 10000'
        WHEN cbc_devices.org_endpoints < 100000 THEN '10000 - 100000'
        ELSE '100000+' END
      AS org_endpoints_bucket,
      COALESCE(cbc_devices.org_windows_endpoints, 0) as org_windows_endpoints,
      COALESCE(cbc_devices.org_mac_endpoints, 0) as org_mac_endpoints,
      COALESCE(cbc_devices.org_linux_endpoints, 0) as org_linux_endpoints,
      COALESCE("Is Customer", false) as "Is Customer",
      account_name,
      account_sfid,
      account_industry,
      sf_link,
      --COALESCE(customers.customer_endpoints, 0) as customer_endpoints,
      --COALESCE(customers.customer_endpoints_consumed, 0) as customer_endpoints_consumed,
      COALESCE("Endpoint Standard Purchased", false) as "Endpoint Standard Purchased",
      COALESCE("Enterprise EDR Purchased", false) as "Enterprise EDR Purchased",
      COALESCE("XDR Purchased", false) as "XDR Purchased",
      COALESCE("Managed Detection Purchased", false) as "Managed Detection Purchased",
      COALESCE("Managed Detection & Response Purchased", false) as "Managed Detection & Response Purchased",
      CASE WHEN "Managed Detection Purchased" OR "Managed Detection & Response Purchased" THEN true ELSE false END as "MD/R Purchased",
      COALESCE("HBFW Purchased", false) as "HBFW Purchased",
      COALESCE("Prevention Purchased", false) as "Prevention Purchased",
      COALESCE("Audit & Remediation Purchased", false) as "Audit & Remediation Purchased",
      COALESCE("Workload Purchased", false) as "Workload Purchased",
      COALESCE("Container Purchased", false) as "Container Purchased",
      most_recent_login
    FROM
      orgs
    LEFT OUTER JOIN customers
    USING(org_key)
    --LEFT OUTER JOIN org_configs
    --USING(org_key)
    LEFT OUTER JOIN cbc_devices
    USING(org_key)
    )