WITH unified_orgs AS (
        SELECT
        -- Get orgs with MDR Unified Console enabled yesterday.
        org_key,
        org_name
        FROM product_metrics.temp_cbc_org_names
)
,mdr_alerts AS (
    WITH mdr_alerts_all AS (
       SELECT
         triggered_alert1_alert_id_guid alert_id
       , environment
       , org_id
       , org_key
       , triggered_alert1_threat_score severity
       , device_name
       , CAST(device_id AS bigint) device_id
       , actor_process_filename app_name
       , (triggered_alert1_alert_creation_timestamp AT TIME ZONE 'Australia/Sydney') create_timestamp
       , "date"(triggered_alert1_alert_creation_timestamp AT TIME ZONE 'Australia/Sydney') create_date,
       CASE
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('TRUE_POSITIVE', 'NOT_ENOUGH_INFO')
           THEN 'Likely Threat'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('FALSE_POSITIVE')
           THEN 'Unlikely threat'
         WHEN (
           triggered_alert[1].user_state_details.workflow_state.status = 'CLOSED'
           AND (
             (triggered_alert[1].mdr_state_details.classification.last_update_timestamp < (DATE '2022-01-01') OR triggered_alert[1].mdr_state_details.classification.last_update_timestamp IS NULL)
             OR (triggered_alert[1].user_state_details.workflow_state.last_update_timestamp <= triggered_alert[1].mdr_state_details.classification.last_update_timestamp)
           )
         )
           THEN 'Closed by customer'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('NOT_REVIEWED')
           THEN 'Not reviewed'
         WHEN triggered_alert1_mdr_state_details_classification_alert_classification IN ('ALERT_CLASSIFICATION_UNKNOWN')
           THEN null
         ELSE null END
       AS determination
       , (triggered_alert[1].mdr_state_details.workflow_state.last_update_timestamp AT TIME ZONE 'Australia/Sydney') as mdr_last_timestamp
       FROM
         (athena.flink_alert_mdr_prodsyd
       LEFT JOIN product_metrics.temp_cbc_org_names USING (org_key))
       WHERE ((triggered_alert1_alert_creation_timestamp AT TIME ZONE 'Australia/Sydney' >= (date_trunc('month', current_timestamp AT TIME ZONE 'Australia/Sydney') - INTERVAL '1' MONTH))
                  AND (triggered_alert1_alert_creation_timestamp AT TIME ZONE 'Australia/Sydney' < date_trunc('month', current_timestamp AT TIME ZONE 'Australia/Sydney')))
    ),
    mdr_alerts_updates AS (
    -- Get Alerts' update
       SELECT
           alert_id,
           CASE
              WHEN mdr_ac IN ('TRUE_POSITIVE', 'NOT_ENOUGH_INFO')
                THEN 'Likely Threat'
              WHEN mdr_ac IN ('FALSE_POSITIVE')
                THEN 'Unlikely threat'
              WHEN (
                user_status = 'CLOSED'
                AND (
                  (mdr_ac_last_timestamp < (DATE '2022-01-01') OR mdr_ac_last_timestamp IS NULL)
                  OR (user_last_timestamp <= mdr_ac_last_timestamp)
                )
              )
                THEN 'Closed by customer'
              WHEN mdr_ac IN ('NOT_REVIEWED')
                THEN 'Not reviewed'
              WHEN mdr_ac IN ('ALERT_CLASSIFICATION_UNKNOWN')
                THEN 'None'
              ELSE 'Unknown' END
            AS determination,
           mdr_workflow_last_timestamp AT TIME ZONE 'Australia/Sydney' AS mdr_last_timestamp
       FROM (
        select
            triggered_alert1_mdr_state_details_classification_alert_classification as mdr_ac,
            triggered_alert[1].mdr_state_details.classification.last_update_timestamp as mdr_ac_last_timestamp,
            triggered_alert[1].user_state_details.workflow_state.status as user_status,
            triggered_alert[1].user_state_details.workflow_state.last_update_timestamp as user_last_timestamp,
            triggered_alert1_alert_id_guid as alert_id,
            triggered_alert[1].mdr_state_details.workflow_state.last_update_timestamp as mdr_workflow_last_timestamp,
                        row_number() over (
            partition by triggered_alert1_alert_id_guid
            order by triggered_alert[1].mdr_state_details.workflow_state.last_update_timestamp desc
       ) as seqnum
                from athena.flink_alert_mdr_prodsyd
                where (event_date >= "substr"(CAST((current_date - INTERVAL  '7' DAY) AS varchar), 1, 10))
                  and triggered_alert[1].mdr_state_details.workflow_state.last_update_timestamp is not null
                  --and triggered_alert[1].mdr_alert=true
       ) where seqnum = 1
    )
    , mdr_alerts AS (
       SELECT
         alert_id
       , environment
       , org_id
       , org_key
       , severity
       , device_name
       , device_id
       , app_name
       , create_timestamp
       , create_date
       , CASE WHEN COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp) IS NOT NULL
                       AND COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp) > create_timestamp
           THEN date_diff('second', create_timestamp, COALESCE(mdr_alerts_updates.mdr_last_timestamp, mdr_alerts_all.mdr_last_timestamp))
           ELSE 0 END AS full_ttr
       , COALESCE(mdr_alerts_updates.determination, mdr_alerts_all.determination, 'None') as determination
       , 'unified' source
       FROM
         mdr_alerts_all LEFT JOIN mdr_alerts_updates USING (alert_id)
    )
    SELECT * FROM mdr_alerts
),
likely_ttr AS (
    -- Get TTR from likely alerts for for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as likely_avg_ttr,
        COUNT(*) as likely_count
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date
),
mdr_ttr AS (
    -- Get TTR from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as mdr_avg_ttr,
        COUNT(*) as mdr_count
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date
),
likely_alert_stat AS (
    -- Get alert severity distribution from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS likely_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS likely_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS likely_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS likely_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS likely_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS likely_alert_10
    FROM mdr_alerts
    WHERE determination = 'Likely Threat'
    GROUP BY org_key, create_date
),
mdr_alert_stat AS (
    -- Get alert severity distribution from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS mdr_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS mdr_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS mdr_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS mdr_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS mdr_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS mdr_alert_10
    FROM mdr_alerts
    WHERE severity >= 5
    GROUP BY org_key, create_date
),
likely_app_freq AS (
    -- Get rank of application and its name from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS likely_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS likely_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS likely_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS likely_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS likely_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS likely_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS likely_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS likely_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS likely_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS likely_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS likely_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS likely_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
mdr_app_freq AS (
    -- Get rank of application and its name from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS mdr_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS mdr_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS mdr_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS mdr_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS mdr_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS mdr_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS mdr_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS mdr_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS mdr_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS mdr_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS mdr_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS mdr_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
likely_dev_freq AS (
        -- Get rank of device and its name from likely alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS likely_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS likely_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS likely_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS likely_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS likely_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS likely_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS likely_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS likely_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS likely_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS likely_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS likely_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS likely_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
),
mdr_dev_freq AS (
        -- Get rank of device and its name from MDR alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS mdr_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS mdr_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS mdr_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS mdr_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS mdr_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS mdr_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS mdr_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS mdr_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS mdr_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS mdr_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS mdr_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS mdr_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
),
industry_comparision AS (
    WITH assets AS (
        -- Get org size and endpoints yesterday
        select orgs.org_key,
        max(registered_total_active) as org_endpoints,
        CASE
            WHEN max(registered_total_active) < 500 THEN 'Small'
            WHEN max(registered_total_active) < 5000 THEN 'Medium'
            ELSE 'Large' END as org_size
        FROM product_metrics.cbc_device_stats devices
        INNER JOIN product_metrics.cbc_orgs orgs
        ON orgs.environment = devices.environment AND orgs.org_id = devices.org_id
        WHERE sh_date = (SELECT max(sh_date) from product_metrics.cbc_device_stats)
        GROUP BY orgs.org_key
    ),
    customer AS (
        -- Get org industry value
        select
            customer_org_key as org_key,
            account_industry,
            org_size,
            org_endpoints
        FROM product_metrics.pum_cbc_report_last_day p
        CROSS JOIN unnest(p.org_key) as t(customer_org_key)
        INNER JOIN assets a ON a.org_key = t.customer_org_key
        WHERE sh_date = (SELECT max(sh_date) from product_metrics.pum_cbc_report_last_day)
    ),
    alerts_per_org_per_day AS (
        -- Calculate alert count for each org every day.
        SELECT
            create_date,
            org_key,
            count(distinct alert_id) as alert_count,
            account_industry,
            org_size,
            max(org_endpoints) as org_endpoints
        FROM mdr_alerts alerts
        LEFT JOIN customer using(org_key)
        GROUP BY create_date, org_key, account_industry, org_size
    ),
    customer_avg AS (
        -- Calculate average alerts per asset per day
        SELECT
            create_date,
            org_key,
            org_size,
            account_industry,
            sum(alert_count) as customer_alert_count,
            cast(sum(alert_count) as double) / cast(max(org_endpoints) + 1 as double) as customer_alerts_per_asset
        FROM alerts_per_org_per_day
        GROUP BY org_key, create_date, org_size, account_industry
    ),
    size_avg AS (
        -- Calculate average alerts per asset per day for each size
        SELECT
            create_date,
            org_size,
            cast(sum(alert_count) as double) / cast(sum(org_endpoints) + 1 as double) as size_alerts_per_asset
        FROM alerts_per_org_per_day
        GROUP BY create_date, org_size
    ),
    industry_avg AS (
        -- Calculate average alerts per asset per day for each industry
        SELECT
            create_date,
            account_industry,
            cast(sum(alert_count) as double) / cast(sum(org_endpoints) + 1 as double) as industry_alerts_per_asset
        FROM alerts_per_org_per_day
        GROUP BY create_date, account_industry
    )
    SELECT *
    FROM customer_avg
    LEFT JOIN size_avg using (create_date, org_size)
    LEFT JOIN industry_avg using (create_date, account_industry)
),
assets AS (
        SELECT
        orgs.org_key,
        environment,
        org_id,
        a.*
    FROM (
        SELECT
            DATE(sh_date) as create_date,
            environment,
            org_id,
            registered,
            registered_total_active,
            registered_active_windows,
            registered_active_mac,
            registered_active_linux
            FROM "product_metrics"."cbc_device_stats_cleansed"
            where CAST(sh_date AS DATE) >= date_trunc('month', current_date - interval '1' month)
            and CAST(sh_date AS DATE) < date_trunc('month', current_date)
        ) AS a
        LEFT JOIN product_metrics.cbc_orgs orgs
        using(environment, org_id)
),
policy AS (
    SELECT org_key,
           CAST(SUM(CASE WHEN quarantine_permission THEN devices_count ELSE 0 END) AS double) / CAST(SUM(devices_count) AS double) AS ratio_quarantine,
           CAST(SUM(CASE WHEN policy_modification_permission THEN devices_count ELSE 0 END) AS double) / CAST(SUM(devices_count) AS double) AS ratio_policy_modification
    FROM product_metrics.pum_policy_report t
    WHERE t.sh_date = (SELECT max(sh_date) FROM product_metrics.pum_policy_report where org_key = t.org_key)
    GROUP BY t.org_key
)
-- Group alert statistic data by org_key and create_date
SELECT
    org_key,
    unified_orgs.org_name,
    COALESCE(create_date, CURRENT_DATE) as create_date,
    likely_alert_stat.*,
    mdr_alert_stat.*,
    COALESCE(lt.likely_avg_ttr, 0) as likely_avg_ttr,
    COALESCE(lt.likely_count, 0) as likely_count,
    COALESCE(mt.mdr_avg_ttr, 0) as mdr_avg_ttr,
    COALESCE(mt.mdr_count, 0) as mdr_count,
    likely_app.*,
    mdr_app_freq.*,
    likely_dev.*,
    mdr_dev_freq.*,
    industry_comparision.*,
    assets.*,
    policy.*
FROM unified_orgs
LEFT JOIN mdr_alert_stat using (org_key)
LEFT JOIN likely_alert_stat using (org_key, create_date)
LEFT JOIN likely_ttr lt using (org_key, create_date)
LEFT JOIN mdr_ttr mt using (org_key, create_date)
LEFT JOIN likely_app_freq likely_app using (org_key, create_date)
LEFT JOIN mdr_app_freq using (org_key, create_date)
LEFT JOIN likely_dev_freq likely_dev using (org_key, create_date)
LEFT JOIN mdr_dev_freq using (org_key, create_date)
LEFT JOIN industry_comparision using (org_key, create_date)
LEFT JOIN assets using (org_key, create_date)
LEFT JOIN policy using (org_key)